%
%  Subject : Approximation of functions using narrow-base trial functions 
%
%  Example Ej_3_1_4
%
%         Wl = Galerkin
%         Nm = piecewise linear
%         see Zienkiewicz & Morgan page 99
% 

% Number of elements to be used
Nx = 10;

% grid points
x  = (0:Nx)'/Nx;

xe = (x(1:end-1)+x(2:end))/2;

% function to be approximated
phi = ffun_Ej_3_1_4(x);

% function used to match end values
%psi = 0*(phi(1)+(phi(end)-phi(1))*xe);

K = zeros(Nx+1,Nx+1);
f = zeros(Nx+1,1);

for l=2:Nx
    psi = (0:2)'/2; 
    xx  = psi*(x(l)-x(l-1))+x(l-1);
    Na = shape_Ej_3_1_4(xx,x(l-1),x(l));
    Nb = shape_Ej_3_1_4(xx,x(l),x(l-1));
    K(l,l-1) = K(l,l-1) + trapz(xx,Nb.*Na);
    K(l,l) = K(l,l) + trapz(xx,Nb.*Nb);    
    xx  = psi*(x(l+1)-x(l))+x(l);
    Na = shape_Ej_3_1_4(xx,x(l+1),x(l));
    Nb = shape_Ej_3_1_4(xx,x(l),x(l+1));
    K(l,l) = K(l,l) + trapz(xx,Na.*Na);
    K(l,l+1) = K(l,l+1) + trapz(xx,Na.*Nb);    

    psi = (0:20)'/20; 
    xx  = psi*(x(l)-x(l-1))+x(l-1);    
    f(l,1) = f(l,1) + trapz(xx,shape_Ej_3_1_4(xx,x(l-1),x(l)).*ffun_Ej_3_1_4(xx));
    xx  = psi*(x(l+1)-x(l))+x(l);    
    f(l,1) = f(l,1) + trapz(xx,shape_Ej_3_1_4(xx,x(l+1),x(l)).*ffun_Ej_3_1_4(xx));    
end

l=Nx+1;
psi = (0:2)'/2; 
xx  = psi*(x(l)-x(l-1))+x(l-1);
Na = shape_Ej_3_1_4(xx,x(l-1),x(l));
Nb = shape_Ej_3_1_4(xx,x(l),x(l-1));
K(l,l-1) = K(l,l-1) + trapz(xx,Nb.*Na);
K(l,l) = K(l,l) + trapz(xx,Nb.*Nb);    

l=1;
xx  = psi*(x(l+1)-x(l))+x(l);
Na = shape_Ej_3_1_4(xx,x(l+1),x(l));
Nb = shape_Ej_3_1_4(xx,x(l),x(l+1));
K(l,l) = K(l,l) + trapz(xx,Na.*Na);
K(l,l+1) = K(l,l+1) + trapz(xx,Na.*Nb);    

l=Nx+1;
psi = (0:20)'/20; 
xx  = psi*(x(l)-x(l-1))+x(l-1);    
f(l,1) = f(l,1) + trapz(xx,shape_Ej_3_1_4(xx,x(l-1),x(l)).*ffun_Ej_3_1_4(xx));
l=1;
xx  = psi*(x(l+1)-x(l))+x(l);    
f(l,1) = f(l,1) + trapz(xx,shape_Ej_3_1_4(xx,x(l+1),x(l)).*ffun_Ej_3_1_4(xx));    

% solver
a = K\f;

% Visualization
figure(1);clf;
plot(x,ffun_Ej_3_1_4(x),'m',xe,zeros(size(xe)),'r+',x,zeros(size(x)),'bo',x,a,'k*')
Xh=[]; Yh=[];
xlabel('X');ylabel('phi');title('1D piecewise linear - Galerkin aproximation ')

for k=1:Nx
    Xh = [Xh,[x(k),x(k+1)]];
    Yh = [Yh,[a(k),a(k+1)]];
end
figure(1);hold on;line(Xh,Yh);

